<?php

use App\Http\Controllers\Admin\AdminController;
use App\Http\Controllers\Admin\AuthController;
use App\Http\Controllers\Admin\Channel\DistributorController;
use App\Http\Controllers\Admin\Finance\StatisticController;
use App\Http\Controllers\Admin\FinanceController;
use App\Http\Controllers\Admin\IndexController;
use App\Http\Controllers\Admin\Order\AlarmController;
use App\Http\Controllers\Admin\Project\CategoryController;
use App\Http\Controllers\Admin\System\MenusController;
use App\Http\Controllers\Admin\System\RegionController;
use App\Http\Controllers\Admin\System\RoleAuthController;
use App\Http\Controllers\Admin\System\UploadController;
use App\Http\Controllers\Admin\SystemController;
use App\Http\Controllers\Admin\Task\AgentStatisticController;
use App\Http\Controllers\Admin\User\ArtificerController;
use App\Http\Controllers\Admin\User\MemberController;
use App\Http\Controllers\Admin\Wechat\OfficialAccountController;
use App\Http\Controllers\Admin\Wechat\WechatController;
use App\Http\Controllers\Auth\EmailVerificationNotificationController;
use App\Http\Controllers\Auth\NewPasswordController;
use App\Http\Controllers\Auth\PasswordResetLinkController;
use App\Http\Controllers\Auth\VerifyEmailController;
use Illuminate\Support\Facades\Route;

Route::prefix('admin')->name('api.admin.')->group(function () {
    Route::middleware('guest')->group(function () {
        Route::post('/register', [AdminController::class, 'store'])->name('register');
        Route::post('/login', [AuthController::class, 'store'])->name('login');

        Route::prefix('task')->group(function () {
            Route::get('/agent/statistic', [AgentStatisticController::class, 'daily']);
        });

        Route::prefix('wechat')->group(function () {
//            Route::match(['get','post'],'/officialAccount', [OfficialAccountController::class, 'serve']);
            Route::get('/officialAccount', [OfficialAccountController::class, 'serve']);
            Route::get('/officialAccount/oauth', [OfficialAccountController::class, 'oauth']);
            Route::get('/officialAccount/oauth/callback', [OfficialAccountController::class, 'oauth_callback']);
            Route::get('/officialAccount/signature', [OfficialAccountController::class, 'signature']);
        });
    });
    Route::middleware('auth.admin')->group(function () {
        Route::post('/logout', [AuthController::class, 'destroy'])->name('logout');
        Route::post('/info', [IndexController::class, 'info'])->name('info');
        Route::put('/refreshToken', [IndexController::class, 'refreshToken'])->name('refreshToken');
        Route::post('/menus', [AuthController::class, 'menus']);

        Route::prefix('order')->name('order.')->group(function () {
            Route::get('/', [\App\Http\Controllers\Admin\Order\IndexController::class, 'index'])->name('index');
            Route::post('/receive', [\App\Http\Controllers\Admin\Order\IndexController::class, 'receive'])->name('receive');
            Route::post('/start', [\App\Http\Controllers\Admin\Order\IndexController::class, 'start'])->name('start');
            Route::post('/reset', [\App\Http\Controllers\Admin\Order\IndexController::class, 'reset'])->name('reset');
            Route::delete('/delete', [\App\Http\Controllers\Admin\Order\IndexController::class, 'destroy'])->name('destroy');
            Route::get('/grab', [\App\Http\Controllers\Admin\Order\IndexController::class, 'grab']);
            Route::post('/over', [\App\Http\Controllers\Admin\Order\IndexController::class, 'over'])->name('over');
            Route::get('/vicinity', [\App\Http\Controllers\Admin\Order\IndexController::class, 'vicinity']);
            Route::get('/new', [\App\Http\Controllers\Admin\Order\IndexController::class, 'new']);
            Route::get('/alarm', [AlarmController::class, 'index']);
            Route::get('/alarm/realtime', [AlarmController::class, 'realtime']);
        });
        Route::prefix('finance')->name('finance.')->group(function () {
            Route::prefix('withdraw')->name('withdraw.')->group(function () {
                Route::get('/', [FinanceController::class, 'withdraw'])->name('index');
                Route::put('/audit', [FinanceController::class, 'audit'])->name('audit');
            });
            Route::get('/withdrawLog', [FinanceController::class, 'withdrawLog'])->name('withdrawLog');
            Route::get('/statistic/income', [StatisticController::class, 'income'])->name('income');
        });

        Route::prefix('project')->name('project.')->group(function () {
//            Route::get('/distributor/all', [DistributorController::class, 'all']);
            Route::get('/list', [\App\Http\Controllers\Admin\Project\IndexController::class, 'index'])->name('index');
            Route::post('/list', [\App\Http\Controllers\Admin\Project\IndexController::class, 'store']);
            Route::get('/list/{id}', [\App\Http\Controllers\Admin\Project\IndexController::class, 'detail']);
            Route::put('/list', [\App\Http\Controllers\Admin\Project\IndexController::class, 'update']);
            Route::delete('/list/{id}', [\App\Http\Controllers\Admin\Project\IndexController::class, 'destroy']);
            Route::put('/status', [\App\Http\Controllers\Admin\Project\IndexController::class, 'status']);

            Route::get('/region/tree/{id}', [\App\Http\Controllers\Admin\Project\IndexController::class, 'regionTree']);
            Route::get('/region/{id}', [\App\Http\Controllers\Admin\Project\IndexController::class, 'region']);
            Route::post('/region', [\App\Http\Controllers\Admin\Project\IndexController::class, 'regionStore']);
            Route::put('/region', [\App\Http\Controllers\Admin\Project\IndexController::class, 'regionUpdate']);
            Route::delete('/region', [\App\Http\Controllers\Admin\Project\IndexController::class, 'regionDestroy']);

            Route::get('/category', [CategoryController::class, 'index'])->name('category');
            Route::get('/category/all', [CategoryController::class, 'all']);
        });

        Route::prefix('system')->name('system.')->group(function () {
//            Route::prefix('withdraw')->name('withdraw.')->group(function (){
//                Route::get('/', [FinanceController::class, 'withdraw'])->name('index');
//                Route::put('/audit', [FinanceController::class, 'audit'])->name('audit');
//            });
            Route::get('/users', [SystemController::class, 'users'])->name('users');
            Route::get('/roles', [SystemController::class, 'roles'])->name('roles');
            Route::get('/roles/auth/{id}', [RoleAuthController::class, 'detail']);
            Route::post('/roles/auth', [RoleAuthController::class, 'store']);

            Route::get('/menus/all', [MenusController::class, 'all']);
            Route::post('/menus', [MenusController::class, 'store']);
            Route::get('/menus', [MenusController::class, 'index']);
            Route::get('/menus/{id}', [MenusController::class, 'detail']);
            Route::put('/menus', [MenusController::class, 'update']);

            Route::get('/region/all', [RegionController::class, 'all']);

            Route::post('/upload', [UploadController::class, 'store']);
        });

        Route::prefix('users')->name('users.')->group(function () {
            Route::get('/member', [MemberController::class, 'index'])->name('members');
            Route::get('/artificer', [ArtificerController::class, 'index'])->name('artificers');
            Route::post('/artificer/audit', [ArtificerController::class, 'audit']);
        });

        Route::prefix('channel')->name('channel.')->group(function () {
            Route::get('/distributor/all', [DistributorController::class, 'all']);
            Route::get('/distributor', [DistributorController::class, 'index'])->name('distributor');
        });


    });

//    Route::resource('users', 'UsersController');
//    Route::get('/users', 'UsersController@index')->name('users.index');
//    Route::get('/users/{user}', 'UsersController@show')->name('users.show');
//    Route::get('/users/create', 'UsersController@create')->name('users.create');
//    Route::post('/users', 'UsersController@store')->name('users.store'); //增
//    Route::get('/users/{user}/edit', 'UsersController@edit')->name('users.edit'); //查
//    Route::patch('/users/{user}', 'UsersController@update')->name('users.update'); //改
//    Route::delete('/users/{user}', 'UsersController@destroy')->name('users.destroy'); //删


    Route::post('/forgot-password', [PasswordResetLinkController::class, 'store'])
        ->middleware('guest')
        ->name('password.email');

    Route::post('/reset-password', [NewPasswordController::class, 'store'])
        ->middleware('guest')
        ->name('password.store');

    Route::get('/verify-email/{id}/{hash}', VerifyEmailController::class)
        ->middleware(['auth', 'signed', 'throttle:6,1'])
        ->name('verification.verify');

    Route::post('/email/verification-notification', [EmailVerificationNotificationController::class, 'store'])
        ->middleware(['auth', 'throttle:6,1'])
        ->name('verification.send');


});

